@@ -4,6 +4,7 @@ use warnings FATAL => 'all';
 package HTML::Tested::Value;
 use base 'Class::Data::Inheritable';
 use HTML::Entities;
+use HTML::Tested::Seal;
 
 __PACKAGE__->mk_classdata('Arg_Names', []);
 
@@ -13,12 +14,12 @@ sub make_args {
 	$class->Arg_Names([ @$an, @names ]);
 }
 
-__PACKAGE__->make_args(qw(default_value is_disabled is_trusted));
+__PACKAGE__->make_args(qw(default_value is_disabled is_trusted is_sealed));
 
 sub arg {
 	my ($self, $parent, $arg_name) = @_;
 	my $fname = $self->name . "_$arg_name";
-	return exists $parent->{$fname} ?
+	return ($parent && exists $parent->{$fname}) ?
 		$parent->{$fname} : $self->{_args}->{$arg_name};
 }
 
@@ -54,27 +55,50 @@ sub encode_value {
 	return encode_entities($val);
 }
 
+sub get_default_value {
+	my ($self, $caller, $id) = @_;
+	my $res = $self->arg($caller, "default_value");
+	return defined($res)
+			?  ref($res) eq 'CODE'
+				? $res->($self, $id, $caller) : $res
+			: '';
+}
+
+sub get_value {
+	my ($self, $caller, $id) = @_;
+	my $n = $self->name;
+	my $val = $caller->$n;
+	return defined($val) ? $val : $self->get_default_value($caller, $id);
+}
+
+sub seal_value {
+	my ($self, $val) = @_;
+	return HTML::Tested::Seal->instance->encrypt($val);
+}
+
 sub render {
 	my ($self, $caller, $stash, $id) = @_;
 	my $res = '';
-	my $n = $self->name;
 	goto OUT if $self->arg($caller, "is_disabled");
 
-	my $val = $caller->$n;
-	if (defined($val)) {
-		$val = $self->encode_value($val)
-			unless $self->arg($caller, "is_trusted");
-	} else {
-		$val = $self->arg($caller, "default_value");
-		$val = $val->($self, $id, $caller)
-			if (defined($val) && ref($val) eq 'CODE');
-		$val = '' unless defined($val);
+	my $val = $self->get_value($caller, $id);
+	if ($self->arg($caller, "is_sealed")) {
+		$val = $self->seal_value($val);
+	} elsif (!$self->arg($caller, "is_trusted")) {
+		$val = $self->encode_value($val);
 	}
+
 	$res = $self->value_to_string($id, $val, $caller);
 OUT:
-	$stash->{$n} = $res;
+	$stash->{ $self->name } = $res;
 }
 
 sub bless_from_tree { return $_[1]; }
 
+sub absorb_one_value {
+	my ($self, $root, $val, @path) = @_;
+	$root->{ $self->name } = $self->arg(undef, "is_sealed")
+		? HTML::Tested::Seal->instance->decrypt($val) : $val;
+}
+
 1;
